function [x,err,f] = nrndn(x0,tol,iprint)
%
%  [x,err,f] = nrndn(x0,tol,iprint)
%
% This script will perform rootfinding by the
% Newton Raphson with Numerical Derivatives method
% for a system of nonlinear algebraic equations.
%
% inputs:
% x0 = initial guess of x, column vector of length n
% tol = RMS tolerance on relative error on x, scalar
% iprint = value of 1 requests iteration information
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = converged solution, column vector of length n
% f = RMS value of f, scalar
% err = RMS value of relative error on x, scalar
%
% example usage:
%
%  [x,err,f] = nrndn([2,2],1.0e-6,1)
%
% Author:  David Keffer 
% original date: May, 1999
% last modified date: March 22, 2013
%

%
%  maximum iterations
%
maxit = 1000;
%
%  system size
%
n = max(size(x0));
%
%  allocate variables
%
Residual = zeros(n,1); 
Jacobian = zeros(n,n);
InvJ = zeros(n,n);
dx = zeros(n,1);
x = zeros(n,1);
xold = zeros(n,1);
%
% extra variables for numerical derivatives
%
xeval = zeros(n,1);
xp = zeros(2);
fp = zeros(n,n,2);
dxcon = zeros(n,1);
%
%  initialize the size of the discretization 
% for numerical differentiation
%
dxcon(1:n) = 1.0e-6;
%
%  initialization
%
x = x0;
err = 100.0;
iter = 0;
%
% convergence loop
%
while ( err > tol ) 
   for j = 1:1:n
   	dx(j) = min(dxcon(j)*x(j),dxcon(j));
      i2dx(j) = 1.0/(2.0*dx(j));
   end
   %
   %  evaluate function
   %
   Residual = funkeval(x);
   %
   %  set up a grid for evaluation of numerical partial derivatives
   %  evaluate function at grid points
   %
   for j = 1:1:n
      for i = 1:1:n
         xeval(i) = x(i);
      end
      xp(1) = x(j) - dx(j);
   	xp(2) = x(j) + dx(j);
      for k = 1:1:2
         xeval(j) = xp(k);
  			fp(:,j,k) = funkeval(xeval);
      end
   end
   %
   %  calculate numerical derivative
   %
   for i = 1:1:n
      for j = 1:1:n
         Jacobian(i,j) = i2dx(j)*( fp(i,j,2) - fp(i,j,1) );
      end
   end
   %
   %  evaluate new estimate of solution
   %
   xold = x;
   invJ = inv(Jacobian);
   deltax = -invJ*Residual;
   for j = 1:1:n
      x(j) = xold(j) + deltax(j);
   end
   %
   %  check for convergence
   %
   iter = iter +1;
   err = sqrt( sum(deltax.^2) /n  ) ;
   f = sqrt(sum(Residual.*Residual)/n);
   if (iprint == 1)
      fprintf (1,'iter = %4i, err = %9.2e f = %9.2e \n ', iter, err, f);
   end
   if ( iter > maxit)
      Residual
      error ('maximum number of iterations exceeded');
   end
end


function f = funkeval(x)
%
% these two lines force a column vector of length n
%
n = max(size(x));
f = zeros(n,1);
%
% EDIT BELOW:
%
%global variables for post processing
%
global pHi
global k1 k2 k3 k4 k5 k6 k7 k8 k9 k10
%Setting input variables for solver
cit3n = x(1);
fe3p = x(2);
ba2p = x(3);
no3n=x(4);
%Known Molarities & Citrate ratio control
fe_tot = 0.12;
ba_tot = 0.01;
no3n_tot=0.36;
ratio=20/13;
h3cit_tot = (fe_tot+ba_tot)*ratio;
%H and OH calculator
pH = pHi;
pOH= 14.0-pH;
H = 10.0^(-pH);
OH = 10.0^(-pOH);
%equillibrium constants
k1=1.17*10^-3;
k2=4.17*10^-5;
k3=1.82*10^-6;
k4=5.00*10^6;
k5=1.58*10^11;
k6=1.00*10^11;
k7=6.17;
k8=56.23;
k9=776.25;
k10=8.7;
%Equation Constants
a = 1.0 + H/k3 + H^2/(k2*k3) + H^3/(k1*k2*k3);
b = H*(k4/k3)+k5;
c = H^2*(k7/(k2*k3))+H*(k8/k3)+k9;
d = 1.0+k6*OH;
e = k10;
%
%Total Cation concentration Balance equations
%
% right hand side of equation
cit_rhs = h3cit_tot;
fe3p_rhs = fe_tot;
ba2p_rhs = ba_tot;
no3n_rhs= no3n_tot;
% left hand side of equation
cit_lhs  = a*cit3n+b*fe3p*cit3n+c*ba2p*cit3n;
fe3p_lhs = d*fe3p+b*fe3p*cit3n;
ba2p_lhs = ba2p+e*ba2p*no3n+c*cit3n*ba2p;
no3n_lhs = no3n+e*ba2p*no3n;
%
% Functions = RHS - LHS = 0
%
f(1) = cit_rhs - cit_lhs;
f(2) = fe3p_rhs - fe3p_lhs;
f(3) = ba2p_rhs - ba2p_lhs;
f(4) = no3n_rhs-no3n_lhs;




